@@ -311,6 +311,14 @@ class OpenStackVolumeResizeShrink(Exception):
     pass
 
 
+class OpenStackImageCreate(Exception):
+    pass
+
+
+class OpenStackVolumeCreate(Exception):
+    pass
+
+
 class OpenStack(OpenStackBase):
 
     def __init__(self, config_dir, **kwargs):
@@ -340,8 +348,7 @@ class OpenStack(OpenStackBase):
                         self.o.stack.delete('--yes', '--wait', name)
 
         def delete_images():
-            for image in self.o.image.list('--private', '--format=value', '-c', 'Name', _iter=True):
-                image = image.strip()
+            for image in self.image_list():
                 if prefix is None or prefix in image:
                     leftovers.append(f'image({image})')
                     self.o.image.delete(image)
@@ -452,6 +459,12 @@ class OpenStack(OpenStackBase):
     def backup_date(self):
         return datetime.datetime.today().strftime('%Y-%m-%d')
 
+    def backup_name_create(self, name):
+        return f'{self.backup_date()}-{name}'
+
+    def backup_extract_name(self, backup):
+        return backup[11:]
+
     def backup_create(self, volumes):
         if len(volumes) == 0:
             volumes = [x.strip() for x in self.o.volume.list('--format=value', '-c', 'Name')]
@@ -466,6 +479,105 @@ class OpenStack(OpenStackBase):
         self._backup_available(volumes, date)
         return count
 
+    def backup_latests(self, backups, names):
+        name2backup = {}
+        for backup in sorted(backups):
+            name = self.backup_extract_name(backup)
+            if name in names:
+                name2backup[name] = backup
+        return name2backup
+
+    def backup_download_volumes(self, volumes):
+        backups = self.backup_latests(self.snapshot_list(), volumes)
+        for volume, backup in backups.items():
+            self.create_volume_from_snapshot(backup, backup)
+            self.image_create_from_volume(backup)
+            self.backup_download_image(volume, backup)
+            self.o.image.delete(backup)
+            self.o.volume.delete(backup)
+
+    def backup_download_images(self, images):
+        for image, backup in self.backup_latests(self.image_list_active(), images).items():
+            self.backup_download_image(image, backup)
+
+    def backup_pathname(self, name):
+        d = f'{self.config_dir}/backups'
+        if not os.path.exists(d):
+            os.makedirs(d)
+        return f'{d}/{name}'
+
+    def backup_is_recent(self, name, recent):
+        pathname = self.backup_pathname(name)
+        if os.path.exists(pathname):
+            day = 60 * 60 * 24
+            return os.path.getmtime(pathname) > (time.time() - recent * day)
+        else:
+            return None
+
+    def backup_only_old(self, names, recent):
+        return [n for n in names if not self.backup_is_recent(n, recent)]
+
+    def glance(self):
+        config = self.config['clouds'][self.args.get('cloud', 'production')]
+        auth = config['auth']
+        if 'user_domain_name' in auth:
+            # OVH
+            e = {
+                'OS_AUTH_URL': auth['auth_url'],
+                'OS_PROJECT_ID': auth.get('project_id'),
+                'OS_PROJECT_NAME': auth.get('project_name'),
+                'OS_USER_DOMAIN_NAME': auth.get('user_domain_name'),
+                'OS_USER_DOMAIN_ID': 'default',
+                'OS_USERNAME': auth.get('username'),
+                'OS_PASSWORD': auth['password'],
+                'OS_REGION_NAME': config['region_name'],
+                'OS_IDENTITY_API_VERSION': str(config.get('identity_api_version')),
+                'OS_INTERFACE': 'public',
+            }
+        else:
+            # Fuga
+            e = {
+                'OS_AUTH_URL': auth['auth_url'],
+                'OS_PROJECT_ID': auth.get('project_id'),
+                'OS_PROJECT_DOMAIN_ID': auth.get('project_domain_id'),
+                'OS_USER_DOMAIN_ID': auth.get('user_domain_id'),
+                'OS_USER_ID': auth.get('user_id'),
+                'OS_USERNAME': auth.get('user_id'),
+                'OS_PASSWORD': auth['password'],
+                'OS_REGION_NAME': config['region_name'],
+                'OS_INTERFACE': config.get('interface'),
+                'OS_IDENTITY_API_VERSION': str(config.get('identity_api_version')),
+            }
+        return sh.glance.bake(
+            _tee=True,
+            _tty_out=False,
+            _out=lambda x: log.info(x.strip()),
+            _err=lambda x: log.info(x.strip()),
+            _env=e)
+
+    @retry(Exception, tries=1)
+    def backup_download_image(self, image, backup):
+        pathname = self.backup_pathname(image)
+        #
+        # this can be replaced once openstackclient is able to download images
+        # without allocating a memory amount that is equal to the size of the
+        # image
+        #
+        tmp = f'{pathname}.tmp'
+        try:
+            self.glance()('image-download', '--file', tmp, self.image_id(backup))
+            os.replace(tmp, pathname)
+        finally:
+            if os.path.exists(tmp):
+                os.remove(tmp)
+        return True
+
+    def backup_download(self, volumes, hosts, recent):
+        if volumes:
+            self.backup_download_volumes(self.backup_only_old(volumes, recent))
+        if hosts:
+            self.backup_download_images(self.backup_only_old(hosts, recent))
+
     def _backup_map(self):
         return dict(self._backup_list())
 
@@ -493,21 +605,32 @@ class OpenStack(OpenStackBase):
             raise OpenStackBackupCreate(progress)
 
     def backup_prune(self, days):
-        before = (datetime.datetime.today() - datetime.timedelta(days)).strftime('%Y-%m-%d')
         count = 0
         for name, status in self._backup_list():
-            if name[:10] > before:
+            if not self.backup_name_is_old(name, days):
                 continue
             self.o.volume.snapshot.delete(name)
             count += 1
         return count
 
+    def backup_name_is_old(self, name, days):
+        before = (datetime.datetime.today() - datetime.timedelta(days)).strftime('%Y-%m-%d')
+        if re.match(r'^\d\d\d\d\-\d\d-\d\d-', name):
+            return name[:10] <= before
+        else:
+            return None
+
+    def create_volume_from_snapshot(self, snapshot, volume):
+        if not self.o.volume.list(
+                '-c', 'Name', '--format=value',
+                '--name', volume).strip() == volume:
+            self.o.volume.create('--snapshot', snapshot, volume)
+
     def volume_prune(self, days):
         r = self.o.volume.snapshot.list(
             '--format=json', '--long', '-c', 'Volume', '--limit', '5000')
         volumes_with_snapshots = set([x["Volume"] for x in json.loads(r.stdout)])
         r = self.o.volume.list('--format=json', '-c', 'Name', '-c', 'ID', '--limit', '5000')
-        before = (datetime.datetime.today() - datetime.timedelta(days)).strftime('%Y-%m-%d')
         info = {
             'no_date_prefix': [],
             'has_snapshots': [],
@@ -515,10 +638,11 @@ class OpenStack(OpenStackBase):
             'recent': [],
         }
         for volume, volume_id in set([(x["Name"], x["ID"]) for x in json.loads(r.stdout)]):
-            if re.match(r'^\d\d\d\d\-\d\d-\d\d-', volume):
+            volume_is_old = self.backup_name_is_old(volume, days)
+            if volume_is_old is not None:
                 if volume_id in volumes_with_snapshots:
                     info['has_snapshots'].append(volume)
-                elif volume[:10] < before:
+                elif volume_is_old:
                     self.o.volume.delete(volume)
                     info['deleted'].append(volume)
                 else:
@@ -550,12 +674,21 @@ class OpenStack(OpenStackBase):
             # using current_volume_id here is not possible unfortunately
             self.o.volume.delete(current_volume)
         else:
-            date = self.backup_date()
-            self.o.volume.set('--name', f'{date}-{current_volume}', current_volume_id)
+            backup_name = self.backup_name_create(current_volume)
+            self.o.volume.set('--name', backup_name, current_volume_id)
         self.o.volume.set('--name', current_volume, volume_id)
         self.o.server.add.volume(host, volume_id)
         self.o.server.start(host)
 
+    def volume_status(self, volume):
+        return self.o.volume.show('--format=value', '-c', 'status', volume).strip()
+
+    @retry(OpenStackVolumeCreate, tries=5)
+    def volume_wait_for_available(self, volume):
+        status = self.volume_status(volume)
+        if status != 'available':
+            raise OpenStackVolumeCreate(f'{volume} status is {status}')
+
     def volume_resize(self, host, volume, size):
         attached = self.o.server.show(
             '--format=yaml', '-c', 'volumes_attached', host).stdout.strip()
@@ -584,8 +717,55 @@ class OpenStack(OpenStackBase):
         self.o.server.start(host)
         return True
 
-    def region_empty(self):
-        volumes = self.o.volume.list()
-        servers = self.o.server.list()
-        images = self.o.image.list('--private')
-        return volumes.strip() == '' and servers.strip() == '' and images.strip() == ''
+    def snapshot_list(self):
+        return [
+            i.strip() for i in self.o.volume.snapshot.list(
+                '--format=value', '-c', 'Name', _iter=True)
+        ]
+
+    def image_list(self):
+        return [
+            i.strip() for i in self.o.image.list(
+                '--private', '--format=value', '-c', 'Name', _iter=True)
+        ]
+
+    def image_list_active(self):
+        r = self.o.image.list('--private', '--format=json', '-c', 'Name', '-c', 'Status')
+        return [x["Name"] for x in json.loads(r.stdout) if x["Status"] == 'active']
+
+    def image_status(self, image):
+        return self.o.image.show('--format=value', '-c', 'status', image).strip()
+
+    def image_id(self, image):
+        return self.o.image.show('--format=value', '-c', 'id', image).strip()
+
+    def image_create_from_volume(self, volume):
+        if volume in self.image_list():
+            return False
+        # open('/dev/tty') can be removed at any point, it is a workaround
+        self.o.image.create('--shared', '--volume', volume, volume,
+                            _in=open('/dev/tty'))
+        self.image_wait_for_active(volume)
+        return True
+
+    @retry(OpenStackImageCreate, tries=14)
+    def image_wait_for_active(self, image):
+        status = self.image_status(image)
+        if status != 'active':
+            raise OpenStackImageCreate(f'{image} status is {status}')
+
+    def image_backup_upload(self, name, pathname):
+        date = self.backup_date()
+        image_name = f'{date}-{name}'
+        if image_name in self.image_list():
+            return False
+        self.o.image.create('--private', '--disk-format=qcow2', '--file', pathname, image_name)
+        return True
+
+    def image_backup_prune(self, names, days):
+        to_delete = []
+        for image in self.image_list():
+            if self.backup_name_is_old(image, days) and self.backup_extract_name(image) in names:
+                to_delete.append(image)
+        if to_delete:
+            self.o.image.delete(*to_delete)
