@@ -3,7 +3,7 @@ def pytest_addoption(parser):
         "--enough-hosts",
         action="store",
 
-        default="bind-host,icinga-host,forum-host",
+        default="bind-host,forum-host",
         help="list of hosts"
     )
     parser.addoption(
