@@ -6,8 +6,7 @@ testinfra_hosts = ['ansible://icinga-host']
 class TestChecks(IcingaHelper):
 
     def test_host(self):
-        r = self.get_client().objects.get('Host', 'website-host')
-        assert r['attrs']['name'] == 'website-host'
+        assert 'website-host' in self.get_hosts(host='website-host')
 
     def test_service_disk(self):
         assert self.is_service_ok('website-host!disk')
