@@ -108,7 +108,7 @@ def test_wazuh_syscheck(host, pytestconfig):
     try:
         w = Wazuh(pytestconfig)
         w.run_syscheck('postfix-host')
-        good_md5 = w.get_syscheck_md5('postfix-host', '/etc/screenrc')
+        good_md5 = w.get_syscheck_md5('postfix-host', '/etc/issue')
         with host.sudo():
             host.run("""
             postsuper -d ALL
@@ -121,12 +121,12 @@ def test_wazuh_syscheck(host, pytestconfig):
             ansible_inventory=host.backend.ansible_inventory)
 
         #
-        # modify /etc/screenrc and verify wazuh sees it
+        # modify /etc/issue and verify wazuh sees it
         #
         with postfix_host.sudo():
-            postfix_host.run("date +%s >> /etc/screenrc")
+            postfix_host.run("date +%s >> /etc/issue")
         w.run_syscheck('postfix-host')
-        bad_md5 = w.get_syscheck_md5('postfix-host', '/etc/screenrc')
+        bad_md5 = w.get_syscheck_md5('postfix-host', '/etc/issue')
         assert good_md5 != bad_md5
 
         #
@@ -137,7 +137,7 @@ def test_wazuh_syscheck(host, pytestconfig):
             with host.sudo():
                 cmd = host.run("""
                 ls /var/spool/postfix/hold
-                grep -qw /etc/screenrc /var/spool/postfix/hold/*
+                grep -qw /etc/issue /var/spool/postfix/hold/*
                 """)
             print(cmd.stdout)
             assert cmd.rc == 0, f'{cmd.stdout} {cmd.stderr}'
