@@ -9,11 +9,12 @@ def get_address(inventory):
         open(vars_dir + '/domain.yml'))['domain']
 
 
-def test_weblate(pytestconfig):
+def test_weblate(request, pytestconfig):
+    certs = request.session.infrastructure.certs()
     # weblate freshly recreated may take few mins to be operationnal
     url = get_address(pytestconfig.getoption("--ansible-inventory"))
     for i in range(60, 0, -1):
-        r = requests.get(url, timeout=5, verify='certs')
+        r = requests.get(url, timeout=5, verify=certs)
         if r.status_code == requests.codes.ok:
             break
         time.sleep(5)
