@@ -4,7 +4,7 @@ import shutil
 import textwrap
 import time
 
-from enough.common.dotenough import DotEnoughLibvirt, DotEnoughOpenStack
+from enough.common.dotenough import DotEnough, DotEnoughLibvirt, DotEnoughOpenStack
 from enough.common import libvirt
 from enough.common.openstack import OpenStack
 from enough import settings
@@ -30,6 +30,7 @@ def prepare_config_dir(domain, enough_dot_dir):
     ---
     certificate_authority: letsencrypt_staging
     """))
+    DotEnough._set_bind_service_group(f'{config_dir}/inventory/services.yml')
     return config_dir
 
 
@@ -131,6 +132,10 @@ class InfrastructureLibvirt(Infrastructure):
 
         dotenough = DotEnoughLibvirt(self.config_dir, self.domain)
         dotenough.ensure()
+        #
+        # backups upload to the cloud and need clouds.yml
+        #
+        shutil.copyfile('tests/clouds.yml', f'{self.all_dir}/clouds.yml')
         shutil.copyfile('inventory/group_vars/all/provision.yml', f'{self.all_dir}/provision.yml')
         open(f'{self.all_dir}/certificate.yml', 'w').write(textwrap.dedent(f"""\
         ---
