@@ -8,6 +8,7 @@ from enough.common.dotenough import (
     Hosts,
     DotEnough,
     DotEnoughOpenStack,
+    DotEnoughLibvirt,
     DotEnoughOpenStackUnknownProvider,
 )
 
@@ -71,6 +72,18 @@ def test_service_add_to_group(tmpdir):
     os.system(f'cat {tmpdir}/services.yml')
 
 
+@pytest.mark.parametrize("cls,driver", (
+    (DotEnoughOpenStack, 'openstack'),
+    (DotEnoughLibvirt, 'libvirt'),
+    ))
+def test_dotenough_populate_infrastructure_driver(tmpdir, cls, driver):
+    d = cls(tmpdir, 'test.com')
+    d.populate_config('ownca')
+    all_dir = f'{d.config_dir}/inventory/group_vars/all'
+    v = yaml.safe_load(open(f'{all_dir}/infrastructure.yml').read())
+    assert v['infrastructure_driver'] == driver
+
+
 #
 # DotEnoughOpenStack
 #
