@@ -1,5 +1,7 @@
 import datetime
 import os
+import sh
+import time
 
 import pytest
 import requests_mock as requests_mock_module
@@ -95,6 +97,12 @@ def test_create_test_subdomain_with_bind(tmpdir, mocker, requests_mock):
 #
 # OpenStack
 #
+@pytest.mark.openstack_integration
+def test_glance(dot_openstack):
+    o = OpenStack(dot_openstack.config_dir)
+    assert len(o.glance()('image-list')) > 0
+
+
 def test_server_ip_in_network_ok(mocker):
     ip = '1.2.3.4'
     mocker.patch('enough.common.openstack.OpenStack.server_port_list',
@@ -129,7 +137,7 @@ def test_backup_create_with_name(openstack_name, dot_openstack, caplog):
     o.o.volume.create('--size=1', openstack_name)
     assert o.backup_create([openstack_name]) == 1
     assert o.backup_create([openstack_name]) == 0
-    available_snapshot = f'AVAILABLE {o.backup_date()}-{openstack_name}'
+    available_snapshot = f'AVAILABLE {o.backup_name_create(openstack_name)}'
     assert available_snapshot in caplog.text
     assert o.backup_prune(0) == 1
     assert o.backup_prune(0) == 0
@@ -171,7 +179,7 @@ def test_backup_create_no_names(openstack_name, dot_openstack, caplog):
     o = OpenStack(settings.CONFIG_DIR)
     o.o.volume.create('--size=1', openstack_name)
     o.backup_create([])
-    available_snapshot = f'AVAILABLE {o.backup_date()}-{openstack_name}'
+    available_snapshot = f'AVAILABLE {o.backup_name_create(openstack_name)}'
     assert available_snapshot in caplog.text
 
 
@@ -195,8 +203,7 @@ def test_openstack_replace_volume(openstack_name, dot_openstack,
     assert openstack_name in o.o.volume.list('--name', openstack_name)
 
     other_volume = f'{openstack_name}_other'
-    date = o.backup_date()
-    backup_volume = f'{date}-{openstack_name}'
+    backup_volume = o.backup_name_create(openstack_name)
 
     o.o.volume.create('--size=1', other_volume)
     o.replace_volume(openstack_name, other_volume, delete_volume=True)
@@ -282,3 +289,111 @@ def test_destroy_volumes_with_same_name(openstack_name, dot_openstack):
     assert 2 == o.o.volume.list().count(volume_name)
     o.destroy_everything(openstack_name)
     assert 0 == o.o.volume.list().count(volume_name)
+
+
+@pytest.mark.openstack_integration
+def test_image_backup_prune(tmpdir, mocker, openstack_name, dot_openstack):
+    pathname = f'{tmpdir}/{openstack_name}.qcow2'
+    sh.qemu_img('create', '-f', 'qcow2', pathname, '1M')
+    o = OpenStack(dot_openstack.config_dir)
+
+    today = o.backup_date()
+
+    old = '2010-01-01'
+    mocker.patch('enough.common.openstack.OpenStack.backup_date', return_value=old)
+    assert o.image_backup_upload(openstack_name, pathname) is True
+
+    mocker.patch('enough.common.openstack.OpenStack.backup_date', return_value=today)
+    assert o.image_backup_upload(openstack_name, pathname) is True
+    assert o.image_backup_upload(openstack_name, pathname) is False
+
+    days = 10
+    o.image_backup_prune([openstack_name], days)
+
+    assert o.image_list() == [o.backup_name_create(openstack_name)]
+
+
+def test_backup_is_recent(dot_openstack):
+    o = OpenStack(dot_openstack.config_dir)
+    name = 'NAME'
+    days = 1
+    backup_pathname = o.backup_pathname(name)
+    assert o.backup_is_recent(name, days) is None
+    open(backup_pathname, 'w').write('SOMETHING')
+    assert o.backup_is_recent(name, days) is True
+    old = time.time() - 2 * days * 60 * 60 * 24
+    os.utime(backup_pathname, (old, old))
+    assert o.backup_is_recent(name, days) is False
+
+
+def test_backup_only_old(dot_openstack):
+    o = OpenStack(dot_openstack.config_dir)
+    new = 'NEW'
+    old = 'OLD'
+    days = 1
+    new_pathname = o.backup_pathname(new)
+    open(new_pathname, 'w').write('SOMETHING')
+    assert o.backup_only_old([old, new], days) == [old]
+
+
+def test_backup_latests():
+    o = OpenStack(settings.CONFIG_DIR)
+    name = 'name1'
+    latest = f'2021-03-02-{name}'
+    assert o.backup_latests(
+        ['ignorethat', latest, f'2020-03-02-{name}', '2021-03-02-name2'],
+        [name]
+    ) == {name: latest}
+
+
+@pytest.mark.openstack_integration
+def test_backup_download_images(tmpdir, openstack_name, dot_openstack):
+    original_pathname = f'{tmpdir}/test.qcow2'
+    sh.qemu_img('create', '-f', 'qcow2', original_pathname, '1M')
+    original_md5 = sh.md5sum(original_pathname).split(' ')[0]
+
+    o = OpenStack(dot_openstack.config_dir)
+
+    assert len(o.image_list_active()) == 0
+    assert o.image_backup_upload(openstack_name, original_pathname) is True
+    assert len(o.image_list_active()) > 0
+    o.backup_download(volumes=[], hosts=[openstack_name], recent=1)
+    backup_pathname = o.backup_pathname(openstack_name)
+    backup_md5 = sh.md5sum(backup_pathname).split(' ')[0]
+
+    assert backup_md5 == original_md5
+
+
+@pytest.mark.openstack_integration
+def test_backup_download_volumes(tmpdir, openstack_name, dot_openstack):
+    o = OpenStack(dot_openstack.config_dir)
+    #
+    # create a small volume
+    #
+    o.o.volume.create('--size=1', openstack_name)
+    #
+    # create a backup
+    #
+    o.backup_create([])
+    #
+    # download the backup
+    #
+    o.backup_download(volumes=[openstack_name], hosts=[], recent=1)
+    backup_pathname = o.backup_pathname(openstack_name)
+    assert os.path.exists(backup_pathname)
+
+
+@pytest.mark.openstack_integration
+def test_image_create_from_volume(openstack_name, dot_openstack):
+    o = OpenStack(dot_openstack.config_dir)
+    o.o.volume.create('--size=1', openstack_name)
+    o.volume_wait_for_available(openstack_name)
+    assert o.image_list() == []
+    o.image_create_from_volume(openstack_name)
+    assert o.image_status(openstack_name) == 'active'
+    li = [
+        i.strip() for i in o.o.image.list(
+            '--shared', '--format=value', '-c', 'Name', _iter=True)
+    ]
+    assert li == [openstack_name]
+    o.o.image.delete(openstack_name)
