@@ -9,7 +9,6 @@ import requests
 import urllib3
 import yaml
 import testinfra
-from tests.infrastructure import get_driver
 
 urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
 
@@ -34,13 +33,13 @@ class IcingaHelper(object):
                 re.search('password = "(.*)"', f.content_string).group(1)
             )
 
-    def get_web_session(self):
+    def get_web_session(self, certs='certs'):
         p = ansible_utils.Playbook('.', '.', [self.inventory])
         username = list(p.get_role_variables('icinga2', 'icingaweb2_user').values())[0]
         password = list(p.get_role_variables('icinga2', 'icingaweb2_user_pass').values())[0]
         address = self.get_address()
         session = requests.Session()
-        session.verify = 'certs'
+        session.verify = certs
 
         r = session.get(f'https://{address}/icingaweb2/authentication/login')
         r.raise_for_status()
@@ -56,14 +55,9 @@ class IcingaHelper(object):
 
     @lru_cache(maxsize=1)
     def get_address(self):
-        if get_driver() == 'openstack':
-            vars_dir = f'{self.inventory}/group_vars/all'
-            host = 'icinga.' + yaml.safe_load(
-                open(vars_dir + '/domain.yml'))['domain']
-        else:
-            host = testinfra.get_host('ansible://icinga-service-group',
-                                      ansible_inventory=self.inventory)
-            host = host.ansible.get_variables()['ansible_host']
+        vars_dir = f'{self.inventory}/group_vars/all'
+        host = 'icinga.' + yaml.safe_load(
+            open(vars_dir + '/domain.yml'))['domain']
         return host
 
     def get_api_url(self):
